'use strict';

exports.__esModule = true;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _EventCell = require('./EventCell');

var _EventCell2 = _interopRequireDefault(_EventCell);

var _domHelpersQueryHeight = require('dom-helpers/query/height');

var _domHelpersQueryHeight2 = _interopRequireDefault(_domHelpersQueryHeight);

var _utilsPropTypes = require('./utils/propTypes');

var _utilsEventLevels = require('./utils/eventLevels');

var _utilsSelection = require('./utils/selection');

var propTypes = {
  segments: _react2['default'].PropTypes.array,
  end: _react2['default'].PropTypes.instanceOf(Date),
  start: _react2['default'].PropTypes.instanceOf(Date),

  titleAccessor: _utilsPropTypes.accessor,
  allDayAccessor: _utilsPropTypes.accessor,
  startAccessor: _utilsPropTypes.accessor,
  endAccessor: _utilsPropTypes.accessor,

  onEventClick: _react2['default'].PropTypes.func
};

var EventRow = _react2['default'].createClass({

  displayName: 'EventRow',

  propTypes: propTypes,

  getDefaultProps: function getDefaultProps() {
    return {
      segments: [],
      selected: [],
      slots: 7
    };
  },

  render: function render() {
    var _this = this;

    var _props = this.props;
    var segments = _props.segments;
    var start = _props.start;
    var end = _props.end;

    var lastEnd = 1;

    return _react2['default'].createElement(
      'div',
      { className: 'rbc-row' },
      segments.reduce(function (row, _ref, li) {
        var event = _ref.event;
        var left = _ref.left;
        var right = _ref.right;
        var span = _ref.span;

        var key = '_lvl_' + li;
        var gap = left - lastEnd;

        var content = _this.renderEvent(event, start, end);

        if (gap) row.push(_this.renderSpan(gap, key + '_gap'));

        row.push(_this.renderSpan(span, key, content));

        lastEnd = right + 1;

        return row;
      }, [])
    );
  },

  renderEvent: function renderEvent(event, start, end) {
    var props = arguments[3] === undefined ? {} : arguments[3];
    var _props2 = this.props;
    var eventPropGetter = _props2.eventPropGetter;
    var selected = _props2.selected;
    var startAccessor = _props2.startAccessor;
    var endAccessor = _props2.endAccessor;
    var titleAccessor = _props2.titleAccessor;
    var eventComponent = _props2.eventComponent;
    var onSelect = _props2.onSelect;

    return _react2['default'].createElement(_EventCell2['default'], {
      event: event,
      eventPropGetter: eventPropGetter,
      onSelect: onSelect,
      selected: _utilsSelection.isSelected(event, selected),
      startAccessor: startAccessor,
      endAccessor: endAccessor,
      titleAccessor: titleAccessor,
      slotStart: start,
      slotEnd: end,
      component: eventComponent
    });
  },

  renderSpan: function renderSpan(len, key) {
    var content = arguments[2] === undefined ? ' ' : arguments[2];
    var slots = this.props.slots;

    return _react2['default'].createElement(
      'div',
      { key: key, className: 'rbc-row-segment', style: _utilsEventLevels.segStyle(Math.abs(len), slots) },
      content
    );
  },

  getRowHeight: function getRowHeight() {
    _domHelpersQueryHeight2['default'](_reactDom.findDOMNode(this));
  }
});

exports['default'] = EventRow;
module.exports = exports['default'];