'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _utilsDates = require('./utils/dates');

var _utilsDates2 = _interopRequireDefault(_utilsDates);

var _localizer = require('./localizer');

var _localizer2 = _interopRequireDefault(_localizer);

var _lodashArrayChunk = require('lodash/array/chunk');

var _lodashArrayChunk2 = _interopRequireDefault(_lodashArrayChunk);

var _lodashObjectOmit = require('lodash/object/omit');

var _lodashObjectOmit2 = _interopRequireDefault(_lodashObjectOmit);

var _utilsConstants = require('./utils/constants');

var _utilsHelpers = require('./utils/helpers');

var _domHelpersQueryHeight = require('dom-helpers/query/height');

var _domHelpersQueryHeight2 = _interopRequireDefault(_domHelpersQueryHeight);

var _domHelpersQueryPosition = require('dom-helpers/query/position');

var _domHelpersQueryPosition2 = _interopRequireDefault(_domHelpersQueryPosition);

var _domHelpersUtilRequestAnimationFrame = require('dom-helpers/util/requestAnimationFrame');

var _domHelpersUtilRequestAnimationFrame2 = _interopRequireDefault(_domHelpersUtilRequestAnimationFrame);

var _EventRow = require('./EventRow');

var _EventRow2 = _interopRequireDefault(_EventRow);

var _Popup = require('./Popup');

var _Popup2 = _interopRequireDefault(_Popup);

var _reactOverlaysLibOverlay = require('react-overlays/lib/Overlay');

var _reactOverlaysLibOverlay2 = _interopRequireDefault(_reactOverlaysLibOverlay);

var _BackgroundCells = require('./BackgroundCells');

var _BackgroundCells2 = _interopRequireDefault(_BackgroundCells);

var _utilsPropTypes = require('./utils/propTypes');

var _utilsEventLevels = require('./utils/eventLevels');

var isSegmentInSlot = function isSegmentInSlot(seg, slot) {
  return seg.left <= slot && seg.right >= slot;
};

var eventsForWeek = function eventsForWeek(evts, start, end, props) {
  return evts.filter(function (e) {
    return _utilsEventLevels.inRange(e, start, end, props);
  });
};

var propTypes = _extends({}, _EventRow2['default'].PropTypes, {

  culture: _react2['default'].PropTypes.string,

  date: _react2['default'].PropTypes.instanceOf(Date),

  min: _react2['default'].PropTypes.instanceOf(Date),
  max: _react2['default'].PropTypes.instanceOf(Date),

  dateFormat: _utilsPropTypes.dateFormat,

  weekdayFormat: _utilsPropTypes.dateFormat,

  onSelectEvent: _react2['default'].PropTypes.func,
  onSelectSlot: _react2['default'].PropTypes.func
});

var MonthView = _react2['default'].createClass({

  displayName: 'MonthView',

  propTypes: propTypes,

  getInitialState: function getInitialState() {
    return {
      rowLimit: 5,
      needLimitMeasure: true
    };
  },

  componentWillMount: function componentWillMount() {
    this._bgRows = [];
    this._pendingSelection = [];
  },

  componentWillReceiveProps: function componentWillReceiveProps(_ref) {
    var date = _ref.date;

    this.setState({
      needLimitMeasure: !_utilsDates2['default'].eq(date, this.props.date)
    });
  },

  componentDidMount: function componentDidMount() {
    var _this = this;

    var running = undefined;

    if (this.state.needLimitMeasure) this._measureRowLimit(this.props);

    window.addEventListener('resize', this._resizeListener = function () {
      if (!running) {
        _domHelpersUtilRequestAnimationFrame2['default'](function () {
          running = false;
          _this.setState({ needLimitMeasure: true });
        });
      }
    }, false);
  },

  componentDidUpdate: function componentDidUpdate(prevProps, prevState) {
    if (this.state.needLimitMeasure) this._measureRowLimit(this.props);
  },

  componentWillUnmount: function componentWillUnmount() {
    window.removeEventListener('resize', this._resizeListener, false);
  },

  render: function render() {
    var _this2 = this;

    var _props = this.props;
    var date = _props.date;
    var culture = _props.culture;
    var weekdayFormat = _props.weekdayFormat;
    var month = _utilsDates2['default'].visibleDays(date, culture);
    var rows = _lodashArrayChunk2['default'](month, 7);

    var measure = this.state.needLimitMeasure;

    this._rowCount = rows.length;

    var elementProps = _lodashObjectOmit2['default'](this.props, Object.keys(propTypes));

    return _react2['default'].createElement(
      'div',
      _extends({}, elementProps, {
        className: _classnames2['default']('rbc-month-view', elementProps.className)
      }),
      _react2['default'].createElement(
        'div',
        { className: 'rbc-row rbc-month-header' },
        this._headers(rows[0], weekdayFormat, culture)
      ),
      rows.map(function (row, idx) {
        return _this2._row(row, idx, measure && _this2._renderMeasureRows);
      })
    );
  },

  _row: function _row(row, rowIdx, content) {
    var _this3 = this;

    var first = row[0];
    var last = row[row.length - 1];
    var evts = eventsForWeek(this.props.events, row[0], row[row.length - 1], this.props);

    evts.sort(function (a, b) {
      return _utilsEventLevels.sortEvents(a, b, _this3.props);
    });

    var segments = evts = evts.map(function (evt) {
      return _utilsEventLevels.eventSegments(evt, first, last, _this3.props);
    });
    var limit = this.state.rowLimit;

    var _eventLevels = _utilsEventLevels.eventLevels(segments, limit);

    var levels = _eventLevels.levels;
    var extra = _eventLevels.extra;

    return _react2['default'].createElement(
      'div',
      { key: 'week_' + rowIdx,
        className: 'rbc-month-row',
        ref: !rowIdx && function (r) {
          return _this3._firstRow = r;
        }
      },
      this.renderBackground(row, rowIdx),
      _react2['default'].createElement(
        'div',
        {
          className: 'rbc-row-content'
        },
        _react2['default'].createElement(
          'div',
          {
            className: 'rbc-row',
            ref: !rowIdx && function (r) {
              return _this3._firstDateRow = r;
            }
          },
          this._dates(row)
        ),
        content ? content(levels, row, rowIdx) : levels.map(function (l, idx) {
          return _this3.renderRowLevel(l, row, idx);
        })
      ),
      this.renderShowMore(segments, extra, row, rowIdx),
      this.props.popup && this._renderOverlay()
    );
  },

  renderRowLevel: function renderRowLevel(segments, week, idx) {
    var first = week[0];
    var last = week[week.length - 1];

    return _react2['default'].createElement(_EventRow2['default'], _extends({}, this.props, {
      eventComponent: this.props.components.event,
      onSelect: this._selectEvent,
      key: idx,
      segments: segments,
      start: first,
      end: last
    }));
  },

  renderBackground: function renderBackground(row, idx) {
    var _this4 = this;

    var self = this;

    function onSelectSlot(_ref2) {
      var start = _ref2.start;
      var end = _ref2.end;

      self._pendingSelection = self._pendingSelection.concat(row.slice(start, end + 1));

      clearTimeout(self._selectTimer);
      self._selectTimer = setTimeout(function () {
        return self._selectDates();
      });
    }

    return _react2['default'].createElement(_BackgroundCells2['default'], {
      container: function () {
        return _reactDom.findDOMNode(_this4);
      },
      selectable: this.props.selectable,
      slots: 7,
      ref: function (r) {
        return _this4._bgRows[idx] = r;
      },
      onSelectSlot: onSelectSlot
    });
  },

  renderShowMore: function renderShowMore(segments, extraSegments, row, rowIdx) {
    var _this5 = this;

    return row.map(function (date, idx) {
      var slot = idx + 1;

      var count = extraSegments.filter(function (seg) {
        return isSegmentInSlot(seg, slot);
      }).length;

      var events = segments.filter(function (seg) {
        return isSegmentInSlot(seg, slot);
      }).map(function (seg) {
        return seg.event;
      });

      var onClick = function onClick() {
        return _this5._showMore(events, date, rowIdx, idx);
      };

      return count ? _react2['default'].createElement(
        'a',
        {
          key: 'sm_' + idx,
          href: '#',
          className: 'rbc-show-more rbc-show-offset-' + slot,
          onClick: onClick
        },
        'show ' + count + ' more'
      ) : false;
    });
  },

  _dates: function _dates(row) {
    var _this6 = this;

    return row.map(function (day, colIdx) {
      var offRange = _utilsDates2['default'].month(day) !== _utilsDates2['default'].month(_this6.props.date);

      return _react2['default'].createElement(
        'div',
        {
          key: 'header_' + colIdx,
          style: _utilsEventLevels.segStyle(1, 7),
          className: _classnames2['default']('rbc-date-cell', {
            'rbc-off-range': offRange,
            'rbc-now': _utilsDates2['default'].eq(day, new Date(), 'day')
          })
        },
        _react2['default'].createElement(
          'a',
          { href: '#', onClick: _this6._dateClick.bind(null, day) },
          _localizer2['default'].format(day, _this6.props.dateFormat, _this6.props.culture)
        )
      );
    });
  },

  _headers: function _headers(row, format, culture) {
    var first = row[0];
    var last = row[row.length - 1];

    return _utilsDates2['default'].range(first, last, 'day').map(function (day, idx) {
      return _react2['default'].createElement(
        'div',
        {
          key: 'header_' + idx,
          className: 'rbc-header',
          style: _utilsEventLevels.segStyle(1, 7)
        },
        _localizer2['default'].format(day, format, culture)
      );
    });
  },

  _renderMeasureRows: function _renderMeasureRows(levels, row, idx) {
    var _this7 = this;

    var first = idx === 0;

    return first ? _react2['default'].createElement(
      'div',
      { className: 'rbc-row' },
      _react2['default'].createElement(
        'div',
        { className: 'rbc-row-segment', style: _utilsEventLevels.segStyle(1, 7) },
        _react2['default'].createElement(
          'div',
          { ref: function (r) {
              return _this7._measureEvent = r;
            }, className: _classnames2['default']('rbc-event') },
          _react2['default'].createElement(
            'div',
            { className: 'rbc-event-content' },
            ' '
          )
        )
      )
    ) : _react2['default'].createElement('span', null);
  },

  _renderOverlay: function _renderOverlay() {
    var _this8 = this;

    var overlay = this.state && this.state.overlay || {};

    return _react2['default'].createElement(
      _reactOverlaysLibOverlay2['default'],
      {
        rootClose: true,
        placement: 'bottom',
        container: this,
        show: !!overlay.position,
        onHide: function () {
          return _this8.setState({ overlay: null });
        }
      },
      _react2['default'].createElement(_Popup2['default'], {
        position: overlay.position,
        events: overlay.events,
        slotStart: overlay.date,
        slotEnd: overlay.end,
        selected: this.props.selected,
        onSelect: this._selectEvent,
        eventPropGetter: this.props.eventPropGetter,
        startAccessor: this.props.startAccessor,
        endAccessor: this.props.endAccessor,
        titleAccessor: this.props.titleAccessor
      })
    );
  },

  _measureRowLimit: function _measureRowLimit(props) {
    var eventHeight = _domHelpersQueryHeight2['default'](this._measureEvent);
    var labelHeight = _domHelpersQueryHeight2['default'](this._firstDateRow);
    var eventSpace = _domHelpersQueryHeight2['default'](this._firstRow) - labelHeight;

    this._needLimitMeasure = false;

    this.setState({
      needLimitMeasure: false,
      rowLimit: Math.max(Math.floor(eventSpace / eventHeight), 1)
    });
  },

  _dateClick: function _dateClick(date) {
    this.clearSelection();
    _utilsHelpers.notify(this.props.onNavigate, [_utilsConstants.navigate.DATE, date]);
  },

  _selectEvent: function _selectEvent() {
    //cancel any pending selections so only the event click goes through.
    this.clearSelection();

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _utilsHelpers.notify(this.props.onSelectEvent, args);
  },

  _selectDates: function _selectDates() {
    var slots = this._pendingSelection.slice();

    this._pendingSelection = [];

    slots.sort(function (a, b) {
      return +a - +b;
    });

    _utilsHelpers.notify(this.props.onSelectSlot, {
      slots: slots,
      start: slots[0],
      end: slots[slots.length - 1]
    });
  },

  _showMore: function _showMore(events, date, row, slot) {
    var cell = _reactDom.findDOMNode(this._bgRows[row]).children[slot];

    if (this.props.popup) {
      var position = _domHelpersQueryPosition2['default'](cell, _reactDom.findDOMNode(this));

      this.setState({
        overlay: { date: date, events: events, position: position }
      });
    }

    _utilsHelpers.notify(this.props.onShowMore, [events, date, slot]);
  },

  clearSelection: function clearSelection() {
    clearTimeout(this._selectTimer);
    this._pendingSelection = [];
  }

});

MonthView.navigate = function (date, action) {
  switch (action) {
    case _utilsConstants.navigate.PREVIOUS:
      return _utilsDates2['default'].add(date, -1, 'month');

    case _utilsConstants.navigate.NEXT:
      return _utilsDates2['default'].add(date, 1, 'month');

    default:
      return date;
  }
};

MonthView.range = function (date, _ref3) {
  var culture = _ref3.culture;

  var start = _utilsDates2['default'].firstVisibleDay(date, culture);
  var end = _utilsDates2['default'].lastVisibleDay(date, culture);
  return { start: start, end: end };
};

exports['default'] = MonthView;
module.exports = exports['default'];
//eslint-disable-line